<?php
/*
Plugin Name: Feed Footer
Plugin URI: http://www.blogclout.com/blog/goodies/feed-footer-plugin/
Description: Add custom footers (copyright, notices, advertisements) to the bottom of your posts in your RSS feed. A special <a href="http://www.blogblout.com/blog" target="blogclout">BlogClout</a> plugin 
Author: Sherief Mursyidi
Version: 1.1
Author URI: http://www.blogclout.com/blog/about-sherief-mursyidi/
-----
-----
*/

$feedfooter = null;

add_action('init', 			'feedfooter_init');
add_action('shutdown', 		'feedfooter_shutdown');
add_action('publish_post', 	'feedfooter_process_new_post');
add_action('publish_page', 	'feedfooter_process_new_post');

add_action('rss_head', 		'feedfooter_initialize_rss_rss1');
add_action('rss2_head', 	'feedfooter_initialize_rss_default');
add_action('rdf_header', 	'feedfooter_initialize_rss_default');
add_action('atom_head', 	'feedfooter_initialize_rss_default');

function feedfooter_init() {
	global $wpdb, $feedfooter;
	$feedfooter = new feedfooter;
	$feedfooter->initialize();

	switch($_REQUEST['feedfooter_action']) {
		case 'debug':
			$feedfooter->debug($_REQUEST['feedfooter_reset_index']);
			exit;
		
		case 'reset_syncing':
			update_option($feedfooter->lastSyncIdOption, '0');
			break;

		case 'reset_sync_limit':
			$maxId = $wpdb->get_var("SELECT ID FROM $wpdb->posts ORDER BY ID DESC LIMIT 1");
			if($maxId === '') $maxId = '0';
			update_option($feedfooter->maxSyncIdOption, $maxId);
			break;
	}

	//add_filter('the_content', 'feedfooter_concat_post_feed', 1);
}

function feedfooter_shutdown() {
	global $feedfooter;
	if(is_object($feedfooter)) $feedfooter->destruct();
}

function feedfooter_initialize_rss_default() {
	global $feedfooter;

	$feedfooter->initializeRss();
	$feedfooter->setRssExcerptOption(0);
	add_filter('the_content', 'feedfooter_concat_rss_feed', 1);
}

function feedfooter_initialize_rss_rss1() {
	global $feedfooter;

	$feedfooter->initializeRss();
	$feedfooter->setRssExcerptOption(1);
	add_filter('the_excerpt_rss', 'feedfooter_concat_rss_ad', 1);
}

function feedfooter_feeds() {
	global $feedfooter;

	$feedfooter->outputHtmlFeeds();
}

function feedfooter_concat_post_feed($content = '') {
	global $feedfooter, $wpdb, $post;
	
	if(is_object($post) && is_object($feedfooter)) $content .= $feedfooter->returnPostFeed($post->ID);

	return $content;
}


function feedfooter_concat_rss_feed($content = '') {
	global $feedfooter, $post;

	if(is_object($feedfooter) && is_object($post)){
		$content .= $feedfooter->returnRssFeed($post->ID, $post->post_date);
	}

	return $content;
}

function feedfooter_process_new_post($postId) {
	global $feedfooter;
	
	$feedfooter->initializeRss();
	$feedfooter->returnRssFeed($postId, date('Y-m-d H:i:s'));
}

class feedfooter {
	var $rssMapTable		= 'feedfooter_rss_map';

	var $lastUpdateOption	= 'feedfooter_last_update';
	var $rssInstalledOption	= 'feedfooter_rss_installed';
	var $rssMaxFeedsOption	= 'feedfooter_rss_max_feeds';
	var $rssIndexOption		= 'feedfooter_rss_index';
	var $lastSyncIdOption	= 'feedfooter_last_sync_post_id';
	var $maxSyncIdOption	= 'feedfooter_max_sync_post_id';

	var $feeds;
	var $postFeeds;
	var $rssActiveFeeds;
	var $rssStoredFeeds;
	var $rssMaxFeeds;
	var $rssCurrentIndex;
	var $rssInstalled;
	var $rssExcerptSetting;
	var $rssRestoreSettings = false;
	
	function feedfooter() {
		global $table_prefix;
		$this->rssMapTable  = $table_prefix.$this->rssMapTable;
	}
	
	function debug($resetIndex = false) {
		global $wpdb;

		if($resetIndex){
			add_option($this->rssIndexOption, '1', 'Stores the index for next rss ad to display.');
			update_option($this->rssIndexOption, '1');
			$this->rssCurrentIndex = 1;
		}

		echo "<b>Last Refresh:</b> "		.get_option($this->lastUpdateOption)	."<br>\n";
		echo "<b>RSS Installed:</b> "		.get_option($this->rssInstalledOption)	."<br>\n";
		echo "<b>RSS Max Feeds:</b> "			.get_option($this->rssMaxAdsOption)		."<br>\n";
		echo "<b>RSS Current Index:</b> "	.get_option($this->rssIndexOption)		."<br><br>\n";

		if($wpdb->get_var("SHOW TABLES LIKE '$this->rssMapTable'") != $this->rssMapTable) {
			echo "Feed Footer RSS map table is <b>not installed</b> (".$this->rssMapTable.")<br><br>\n";
		}else{
			echo "Feed Footer map table is installed (".$this->rssMapTable.")<br><br>\n";
			print_r($wpdb->get_results("SELECT * FROM `$this->rssMapTable` ORDER BY post_id DESC LIMIT ".get_option('posts_per_rss')));
			echo "<br><br>";
		}
	}
	
	function installDatabase() {
		global $wpdb;

		require_once(ABSPATH . 'wp-admin/upgrade-functions.php');

		$sql = "CREATE TABLE `$this->rssMapTable` (
					`post_id` INT( 10 ) UNSIGNED NOT NULL DEFAULT '0',
					`feeds` TEXT NOT NULL ,
					PRIMARY KEY ( `post_id` )
				) TYPE = MYISAM ;";

		dbDelta($sql);

		add_option($this->rssInstalledOption, date('Y-m-d H:i:s'), 'Stores the date that Feed Footer rss was installed.');
		add_option($this->lastUpdateOption, '0000-00-00 00:00:00', 'Stores the date of the last Feed Footer plugin data update.');
		add_option($this->rssMaxFeedsOption, '10', 'Stores the number of rss feeds in rotation.');
		add_option($this->rssIndexOption, '1', 'Stores the index for next rss feed to display.');

		$maxId = $wpdb->get_var("SELECT ID FROM $wpdb->posts ORDER BY ID DESC LIMIT 1");
		if($maxId === '') $maxId = '0';

		add_option($this->lastSyncIdOption, '0', 'The ID of the last post synced with Feed Footer');
		add_option($this->maxSyncIdOption, $maxId, 'The highest post ID to be batch synced with Feed Footer');
	}
	
	function initialize() {
		global $wpdb;

		if($wpdb->get_var("SHOW TABLES LIKE '$this->rssMapTable'") != $this->rssMapTable) {
			$this->installDatabase();
		}
	}
	
	function initializeRss() {
		$this->rssInstalled 	= get_option($this->rssInstalledOption);
		$this->rssMaxFeeds 		= get_option($this->rssMaxFeedsOption);
		$this->rssCurrentIndex 	= get_option($this->rssIndexOption);
		
		if($this->rssCurrentIndex === false){
			add_option($this->rssIndexOption, '1', 'Stores the index for next rss feed to display.');
			$this->rssCurrentIndex = 1;
		}

		$this->loadStoredRssFeeds();
		
		remove_filter('the_content', 'feedfooter_concat_post_feed');
	}
	
	function destruct() {
		if($this->rssRestoreSettings){
			update_option('rss_use_excerpt', $this->rssExcerptSetting);
		}
	}
	
	function setRssExcerptOption($val = 0) {
		$this->rssRestoreSettings = true;
		$this->rssExcerptSetting = get_option('rss_use_excerpt');
		update_option('rss_use_excerpt', $val);
	}
	
	function loadStoredRssFeeds() {
		global $wpdb;

		$this->rssStoredFeeds = array();

		$rssFeeds = $wpdb->get_results("SELECT * FROM `$this->rssMapTable` ORDER BY post_id DESC LIMIT ".get_option('posts_per_rss'));

		if(is_array($rssFeeds)) foreach($rssFeeds as $feed) $this->rssStoredFeeds[$feed->post_id] = $feed->feeds;
	}
	
	function returnRssFeed($postId, $postDate) {
		global $wpdb;
		
		if($postDate < $this->rssInstalled) return;

		$returnFeed = '';
		
		if( isset($this->rssStoredFeeds[$postId]) ){
			$returnFeed = $this->rssStoredFeeds[$postId];
		}else if($this->rssMaxFeeds > 0 && $this->rssMaxFeeds != ''){
			$this->rssCurrentIndex = $this->rssCurrentIndex % $this->rssMaxFeeds;
			
			$options = (array) get_settings('feedfooter-params');
			$returnFeed = !empty($options['feedfooter'.$this->rssCurrentIndex]) ? $options['feedfooter'.$this->rssCurrentIndex] : '';

			$result = $wpdb->query("INSERT IGNORE INTO `$this->rssMapTable` SET post_id = '$postId', feeds = '".mysql_real_escape_string($returnFeed)."'");

			if($result === 1){
				$this->rssStoredFeeds[$postId] = $returnFeed;

				$this->rssCurrentIndex++;
				$this->rssCurrentIndex = $this->rssCurrentIndex % $this->rssMaxFeeds;
			}

			update_option($this->rssIndexOption, $this->rssCurrentIndex);
		}
		
		return ($returnFeed != '' ? "<p>$returnFeed</p>" : '');
	}
}

if ( strpos($_SERVER['REQUEST_URI'], 'wp-admin') !== false ) {
	include_once dirname(__FILE__) . '/feed-footer-admin.php';
}
?>